@@ -1,4 +1,6 @@
 # -*- coding: utf-8 -*-
+#
+# Many tests were converted from the Python 2.7 standard library test suite.
 
 import unittest
 
@@ -28,12 +30,10 @@ class BaseFormatterTest(unittest.TestCase):
         # test both with and without the trailing 's'
         self.assertEqual(_strformat(value, fmt), expected)
         self.assertEqual(_strformat(value, fmt + 's'), expected)
-        return expected, fmt, value
 
     def _check_format(self, expected, fmt, *args, **kwargs):
         fmt, expected = self._prepare(fmt, expected)
         self.assertEqual(f(fmt).format(*args, **kwargs), expected)
-        return expected, fmt, args, kwargs
 
     def _check_raises(self, expected_exception, fmt, *args, **kwargs):
         fmt = self._prepare(fmt)
@@ -44,11 +44,11 @@ class BaseFormatterTest(unittest.TestCase):
         else:
             raise self.failureException('%s not raised' %
                                         expected_exception.__name__)
-        return expected_exception, fmt, args, kwargs
 
     assert_raises_25 = _check_raises
 
     def test_strformat(self):
+        # from Python 2.7 test suite
         test = self._check_strformat
 
         # def test(expected, fmt, value):
@@ -80,8 +80,8 @@ class BaseFormatterTest(unittest.TestCase):
         test(' ' * 10000000, '10000000', '')
 
     def test_format(self):
+        # from Python 2.7 test suite
         test = self._check_format
-        assert_raises = self._check_raises
 
         # Safety check
         self.assertTrue(hasattr(f(''), 'format'))
@@ -108,7 +108,6 @@ class BaseFormatterTest(unittest.TestCase):
 
         # classes we'll use for testing
         class C:
-
             def __init__(self, x=100):
                 self._x = x
 
@@ -116,7 +115,6 @@ class BaseFormatterTest(unittest.TestCase):
                 return spec
 
         class D:
-
             def __init__(self, x):
                 self.x = x
 
@@ -125,7 +123,6 @@ class BaseFormatterTest(unittest.TestCase):
 
         # class with __str__, but no __format__
         class E:
-
             def __init__(self, x):
                 self.x = x
 
@@ -134,7 +131,6 @@ class BaseFormatterTest(unittest.TestCase):
 
         # class with __repr__, but no __format__ or __str__
         class F:
-
             def __init__(self, x):
                 self.x = x
 
@@ -143,7 +139,6 @@ class BaseFormatterTest(unittest.TestCase):
 
         # class with __format__ that forwards to string, for some format_spec's
         class G:
-
             def __init__(self, x):
                 self.x = x
 
@@ -157,17 +152,14 @@ class BaseFormatterTest(unittest.TestCase):
 
         # class that returns a bad type from __format__
         class H:
-
             def __format__(self, format_spec):
                 return 1.0
 
         class I(datetime.date):
-
             def __format__(self, format_spec):
                 return self.strftime(str(format_spec))
 
         class J(int):
-
             def __format__(self, format_spec):
                 return int.__format__(self * 2, format_spec)
 
@@ -277,56 +269,95 @@ class BaseFormatterTest(unittest.TestCase):
         test('hello     ', '{0:{width}.{precision}s}',
              'hello world', width='10', precision='5')
 
+    def test_format_auto_numbering(self):
+        # from Python 2.7 test suite
+        test = self._check_format
+
+        class C:
+            def __init__(self, x=100):
+                self._x = x
+
+            def __format__(self, spec):
+                return spec
+
+        test('10', '{}', 10)
+        test('s    ', '{:5}', 's')
+        test("'s'", '{!r}', 's')
+        test('10', '{._x}', C(10))
+        test('2', '{[1]}', [1, 2])
+        test('4', '{[a]}', {'a': 4, 'b': 2})
+        test('a0b1c', 'a{}b{}c', 0, 1)
+
+        test('a    x     b', 'a{:{}}b', 'x', '^10')
+        test('a0x14b', 'a{:{}x}b', 20, '#')
+
+        # can mix and match auto-numbering and named
+        test('test4', '{f}{}', 4, f='test')
+        test('4test', '{}{f}', 4, f='test')
+        test(' 1g3', '{:{f}}{g}{}', 1, 3, g='g', f=2)
+        test(' 14g', '{f:{}}{}{g}', 2, 4, f=1, g='g')
+
+    def test_format_errors(self):
+        # from Python 2.7 test suite
+        assert_raises = self._check_raises
+
         # test various errors
-        #self.assertRaises(ValueError, f('{').format)
-        #self.assertRaises(ValueError, f('}').format)
-        #self.assertRaises(ValueError, f('a{').format)
-        #self.assertRaises(ValueError, f('a}').format)
-        #self.assertRaises(ValueError, f('{a').format)
-        #self.assertRaises(ValueError, f('}a').format)
-        #self.assertRaises(IndexError, f('{0}').format)         # KeyError
-        #self.assertRaises(IndexError, f('{1}').format, 'abc')  # KeyError
+        # XXX failing tests are commented
+        # assert_raises(ValueError, '{')
+        # assert_raises(ValueError, '}')
+        # assert_raises(ValueError, 'a{')
+        # assert_raises(ValueError, 'a}')
+        # assert_raises(ValueError, '{a')
+        # assert_raises(ValueError, '}a')
+        # assert_raises(IndexError, '{0}')          # KeyError
+        # assert_raises(IndexError, '{1}', 'abc')   # KeyError
         assert_raises(KeyError, '{x}')
-        #self.assertRaises(ValueError, f("}{").format)
-        #self.assertRaises(ValueError, f("abc{0:{}").format)    # KeyError
-        #self.assertRaises(ValueError, f("{0").format)
-        #self.assertRaises(IndexError, f("{0.}").format)        # ValueError
+        # assert_raises(ValueError, '}{')
+        # assert_raises(ValueError, 'abc{0:{}')     # KeyError
+        # assert_raises(ValueError, '{0')
+        # assert_raises(IndexError, '{0.}')         # ValueError
         assert_raises(ValueError, '{0.}', 0)
-        #self.assertRaises(IndexError, f("{0[}").format)        # ValueError
-        #XXX assert_raises(ValueError, '{0[}', [])
+        # assert_raises(IndexError, '{0[}')         # ValueError
+        # assert_raises(ValueError, '{0[}', [])
         assert_raises(KeyError, '{0]}')
         assert_raises(ValueError, '{0.[]}', 0)
         assert_raises(ValueError, '{0..foo}', 0)
         assert_raises(ValueError, '{0[0}', 0)
         assert_raises(ValueError, '{0[0:foo}', 0)
         assert_raises(KeyError, '{c]}')
-        #self.assertRaises(ValueError, f("{{ {{{0}}").format, 0)
-        #self.assertRaises(ValueError, f("{0}}").format, 0)
+        # assert_raises(ValueError, '{{ {{{0}}', 0)
+        # assert_raises(ValueError, '{0}}', 0)
         assert_raises(KeyError, '{foo}', bar=3)
-        #self.assertRaises(ValueError, f("{0!x}").format, 3)
-        #self.assertRaises(ValueError, f("{0!}").format, 0)
-        #self.assertRaises(ValueError, f("{0!rs}").format, 0)
-        #self.assertRaises(ValueError, f("{!}").format)         # KeyError
-        # self.assertRaises(IndexError, f("{:}").format)        # KeyError
-        # self.assertRaises(IndexError, f("{:s}").format)       # KeyError
-        # self.assertRaises(IndexError, f("{}").format)         # KeyError
+        assert_raises(ValueError, '{0!x}', 3)
+        assert_raises(ValueError, '{0!}', 0)
+        assert_raises(ValueError, '{0!rs}', 0)
+        assert_raises(ValueError, '{!}')
+        # assert_raises(IndexError, '{:}')          # KeyError
+        # assert_raises(IndexError, '{:s}')         # KeyError
+        # assert_raises(IndexError, '{}')           # KeyError
 
         # issue 6089
-        #self.assertRaises(ValueError, f("{0[0]x}").format, [None])
-        #self.assertRaises(ValueError, f("{0[0](10)}").format, [None])
+        assert_raises(ValueError, '{0[0]x}', [None])
+        assert_raises(ValueError, '{0[0](10)}', [None])
 
         # can't have a replacement on the field name portion
         assert_raises(TypeError, '{0[{1}]}', 'abcdefg', 4)
 
         # exceed maximum recursion depth
-        #self.assertRaises(ValueError, f("{0:{1:{2}}}").format, 'abc', 's', '')
-        #self.assertRaises(ValueError, f("{0:{1:{2:{3:{4:{5:{6}}}}}}}").format,
-                          #0, 1, 2, 3, 4, 5, 6, 7)
+        # assert_raises(ValueError, '{0:{1:{2}}}', 'abc', 's', '')
+        # assert_raises(ValueError, '{0:{1:{2:{3:{4:{5:{6}}}}}}}',
+        #               0, 1, 2, 3, 4, 5, 6, 7)
 
         # string format spec errors
-        #self.assertRaises(ValueError, f("{0:-s}").format, '')
-        #self.assertRaises(ValueError, format, "", "-")
-        #self.assertRaises(ValueError, f("{0:=s}").format, '')
+        assert_raises(ValueError, '{0:-s}', '')
+        self.assertRaises(ValueError, _strformat, "", "-")
+        assert_raises(ValueError, '{0:=s}', '')
+
+        # can't mix and match numbering and auto-numbering
+        assert_raises(ValueError, '{}{1}', 1, 2)
+        assert_raises(ValueError, '{1}{}', 1, 2)
+        assert_raises(ValueError, '{:{1}}', 1, 2)
+        assert_raises(ValueError, '{0:{}}', 1, 2)
 
     def test_incompatibilities(self):
         # Differences with Python 2.7
@@ -335,7 +366,6 @@ class BaseFormatterTest(unittest.TestCase):
         self.assert_raises_25(KeyError, '{0}')
         self.assert_raises_25(KeyError, '{1}', 'abc')
         self.assert_raises_25(KeyError, 'abc{0:{}')     # ValueError
-        self.assert_raises_25(KeyError, '{!}')          # ValueError
         self.assert_raises_25(KeyError, '{:}')
         self.assert_raises_25(KeyError, '{:s}')
         self.assert_raises_25(KeyError, '{}')
@@ -433,7 +463,7 @@ if has_object_format:
         """
 
         # Monkey patch the _format_field to skip builtin method __format__
-        def _format_field(value, parts, conv, spec):
+        def _format_field(value, parts, conv, spec, want_bytes=False):
             for k, part, _ in parts:
                 if k:
                     if part.isdigit():
@@ -451,6 +481,8 @@ if has_object_format:
             else:
                 # Skip the __format__ method for builtin types
                 value = _strformat(value, spec)
+            if want_bytes and isinstance(value, unicode):
+                return str(value)
             return value
 
         # Monkey patch the _format_field to skip builtin method __format__
@@ -519,6 +551,37 @@ class StringFormatterTest(BaseFormatterTest):
 class UnicodeFormatterTest(BaseFormatterTest):
     type2test = unicode
 
+    def test_mixed_unicode_str(self):
+        # from Python 2.7 test suite
+
+        def test(expected, fmt, *args, **kwargs):
+            self.assertEqual(f(fmt).format(*args, **kwargs), expected)
+
+        # test mixing unicode and str
+        self.assertEqual(_strformat(u'abc', 's'), u'abc')
+        self.assertEqual(_strformat(u'abc', '->10s'), u'-------abc')
+
+        # test combining string and unicode
+        test(u'foobar', u"foo{0}", 'bar')
+
+        # This will try to convert the argument from unicode to str, which
+        #  will succeed
+        test('foobar', "foo{0}", u'bar')
+        # This will try to convert the argument from unicode to str, which
+        #  will fail
+        self.assertRaises(UnicodeEncodeError, f("foo{0}").format, u'\u1000bar')
+
+    if has_object_format:   # specific to Python >= 2.6
+        def test_format_subclass(self):
+            # from Python 2.7 test suite
+            class U(unicode):
+                def __unicode__(self):
+                    return u'__unicode__ overridden'
+            u = U(u'xxx')
+
+            self.assertEqual("%s" % u, u'__unicode__ overridden')
+            self.assertEqual(f("{}").format(u), '__unicode__ overridden')
+
 
 def suite():
     suite = unittest.TestSuite()
